# -*- coding: utf-8 -*-
# @Author: Muhammad Umair
# @Date:   2023-01-08 12:26:09
# @Last Modified by:   Muhammad Umair
# @Last Modified time: 2023-01-09 11:51:25


from .watson import Watson
from .lm import WatsonLMInterface
from .am import WatsonAMInterface

