# -*- coding: utf-8 -*-
# @Author: Muhammad Umair
# @Date:   2023-07-04 15:35:16
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-02-06 15:43:20
# @Description: Manage backend configuration files

import os

PROJECT_ROOT = os.path.dirname(os.path.dirname(__file__))
