import os
SUITE_MANAGER_TEST_PATH = os.path.dirname(__file__)
SUITE_TEST_WORKSPACE_PATH = os.path.join(SUITE_MANAGER_TEST_PATH, "suite_workspace")
SUITE_TEST_SRC_PATH = os.path.join(SUITE_MANAGER_TEST_PATH, "suite_src")
