# -*- coding: utf-8 -*-
# @Author: Vivian Li
# @Date:   2024-01-29 13:26:17
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-02-06 15:50:33
# @Implement difference plugin suite loader classes to load plugin suite
# from different kind of sources
from .directoryloader import PluginDirectorySuiteLoader
from .pluginUrlLoader import PluginURLSuiteLoader
from .pluginSuiteLoader import PluginSuiteLoader
