# -*- coding: utf-8 -*-
# @Author: Muhammad Umair
# @Date:   2022-08-23 09:38:56
# @Last Modified by:   Vivian Li
# @Last Modified time: 2024-04-05 08:05:03
import setuptools

if __name__ == "__main__":
    setuptools.setup()
